% Script for Phase Transition Plot resp Time
clear, clc

%% Add Mosek!

%%

M = 10;         % Number of repetions in Monte Carlo experiment
n = 1600;       % Number of Variables aka Number of Rows
delta = floor(double(n*(0:.025:1))); delta = delta(2:end-1); % Array of Number of Equations aka of Columns

% Initialization of Counting Arrays
PT_L1 = zeros(39);
PT_RS = zeros(39);
Time_L1 = zeros(39);
Time_RS = zeros(39);
SA_L1 = zeros(39);
SA_RS = zeros(39);

o = 1; % Initialization of Column Position
for m = delta

    rho = floor(double(m*(0:.025:1))); rho = rho(2:end-1); % Array of Sparsity
    l = 39; % Initialization of Row Position
    
    for k = rho
        
        % Initialization of Counting Variables
        time_l1 = 0;
        time_rs = 0;
        pt_l1 = 0;
        pt_rs = 0;
        sa_l1 = 0;
        sa_rs = 0;
                
        for i = 1:M
            
            A = randn(m,n); % Choose Matrix
            p = randperm(n);
            I = sort(p(1:k)); % Choose Support

            x = zeros(n,1);
            x(I) = randn(k,1); % Choose Vector
            b = A*x; % Compute right-hand side
            s = sign(x); % Compute Signum of x
            
            % % Recoverability Check
            tic
            rankAI = rank(A(:,I));
            time_Rank = toc;
                
            if rankAI == k
                [~,status_inf,norm_y_inf,time_inf] = get_source_element_l_inf(A,s);

                % Increase if status is 'OPTIMAL'
                if strcmp(status_inf,'OPTIMAL')
                    sa_rs = sa_rs + 1/M;
                end
                
                % Increse if instance is Recoverbable Support
                if norm_y_inf < 1 - 1e-12
                    pt_rs = pt_rs + 1/M;
                end

            else
                time_inf = 0;
            end

            % Increase Time
            time_rs = time_rs + (time_inf+time_Rank)/M;
            
            % % L1 Check
            [x_L1,~,status_l1,tme_L1] = solve_l1_minimization(A,b);

            % Increase Time
            time_l1 = time_l1 + tme_L1/M;

            % Increase if solution is x
            if norm(x_L1 - x) < 1e-5
                pt_l1 = pt_l1 + 1/M;
            end

            % Increase if status is 'OPTIMAL'
            if strcmp(status_l1,'OPTIMAL')
                sa_l1 = sa_l1 + 1/M;
            end    

        end
        
        % Update Counting Arrays
        Time_RS(l,o) = time_rs;
        Time_L1(l,o) = time_l1;
        PT_RS(l,o) = pt_rs;
        PT_L1(l,o) = pt_l1;       
        SA_RS(l,o) = sa_rs;
        SA_L1(l,o) = sa_l1;

        
        l = l - 1; % Update Row Position
    end
    o = o + 1; % Update Column Position
    disp(o)
end
