clear, clc, close all

%% Add Mosek!

%%

M = 1000; % Number of Repetitions in MC experiment
n = 15; % Choose Number of Variables aka Number of Columns;

MBF = constructing_mercedes_benz_frame(n-1);

PT_MBF = zeros(n-1,1);
PT_RAND = zeros(n-1,1);
SA_MBF = zeros(n-1,1);
SA_RAND = zeros(n-1,1);
HUpperBound = zeros(n-1,1);


for k = 1:n-1
    for i = 1:M
        
        RAND = randn(n-1,n);
        p = randperm(n);
        I = sort(p(1:k));
        
        rankRAND = rank(RAND(:,I));
        rankMBF = rank(MBF(:,I));
        
        x = zeros(n,1);
        x(I) = randn(k,1);
        s = sign(x);
        
        if rankRAND == k
            [~,status_RAND,norm_RAND,~] = get_source_element_l_inf(RAND,s);
    
            if strcmp(status_RAND,'OPTIMAL')
                SA_RAND(k) = SA_RAND(k) + 1/M;
            end
            
            if norm_RAND < 1 - 1e-12
                PT_RAND(k) = PT_RAND(k) + 1/M;
            end
        end
                
        if rankMBF == k
            [~,status_MBF,norm_MBF,~] = get_source_element_l_inf(MBF,s);
    
            if strcmp(status_MBF,'OPTIMAL')
                SA_MBF(k) = SA_MBF(k) + 1/M;
            end
            
            if norm_MBF < 1 - 1e-12
                PT_MBF(k) = PT_MBF(k) + 1/M;
            end
        end 
    end
    
    HUpperBound(k) = min(2^k*nchoosek(n,k),2^(k+1-n)*(n-floor(n/2))*nchoosek(n,floor(n/2))*nchoosek(n-1,k))/(2^k*nchoosek(n,k));
end

% Plot
EmpBound_MBF = find(PT_MBF < 1 - 1e-12);
EmpBound_MBF = EmpBound_MBF(1)-1;

EmpBound_RAND = find(PT_RAND < 1 - 1e-12);
EmpBound_RAND = EmpBound_RAND(1)-1;

EmpBound = min([EmpBound_MBF, EmpBound_RAND, floor(.5*n)]);

fig1 = figure(1);
hold on
plot([EmpBound:n-1],PT_MBF(EmpBound:n-1),'LineWidth',2)
plot([EmpBound:n-1],HUpperBound(EmpBound:n-1),'r','LineWidth',2);
plot([EmpBound:n-1],PT_RAND(EmpBound:n-1),'c','LineWidth',2)
    
plot(EmpBound_MBF,1,'o','LineWidth',2,...
                'MarkerEdgeColor','b',...
                'MarkerSize',6)

plot(EmpBound_RAND,1,'o','LineWidth',2,...
                'MarkerEdgeColor','c',...
                'MarkerSize',6)
            
plot(floor(.5*n),1,'X','LineWidth',2,...
                'MarkerEdgeColor','k',...
                'MarkerSize',8)

xlabel('Size k')
ylabel('Recoverability in Percent')

axis([EmpBound n-1 0 1.1])

YTick = get(gca,'YTickLabel');
YTickSize = size(YTick,1);

for i = 1:YTickSize
    a = 100*str2num(YTick(i,:));
    if a > 100
        YTick2{i} = '';
    else
        YTick2{i} = num2str(a);
    end
    
end

switch YTickSize
    case 11
        set(gca,'YTickLabel',{YTick2{1},YTick2{2},YTick2{3},YTick2{4},YTick2{5},YTick2{6},YTick2{7},YTick2{8},YTick2{9},YTick2{10},YTick{11}})
    case 10
        set(gca,'YTickLabel',{YTick2{1},YTick2{2},YTick2{3},YTick2{4},YTick2{5},YTick2{6},YTick2{7},YTick2{8},YTick2{9},YTick2{10}})
    case 9
        set(gca,'YTickLabel',{YTick2{1},YTick2{2},YTick2{3},YTick2{4},YTick2{5},YTick2{6},YTick2{7},YTick2{8},YTick{9}})
    case 8
        set(gca,'YTickLabel',{YTick2{1},YTick2{2},YTick2{3},YTick2{4},YTick2{5},YTick2{6},YTick2{7},YTick2{8}})
    case 6
        set(gca,'YTickLabel',{YTick2{1},YTick2{2},YTick2{3},YTick2{4},YTick2{5},YTick2{6}})
      
end

legend('Mercedes-Benz','Upper Bound','Gaussian', 'Empirical Bound Mercedes-Benz', 'Empirical Bound Gaussian','Bound Mutual Coherence',3);
hold off
