function [y,status,norm_y,time] = get_source_element_l_inf(A,s)
% Function get_source_element_l_inf(A,s)
% delivers y satisyfing
% min_y norm(A(:,Ic)'*y,inf) subject to A(:,I)'*y = s,
% with I = find(s~=0), Ic = find(s == 0),
% using Mosek through linear optimization.
%
% Input:  A	: Matrix of l1 minimization problem
%	  s	: Sign of the desired solution x^*
% Output: y	: ArgMin
% 	  status: Status of calculated solution (Mosek)
% 	  norm_y: Min
%	  time	: Time Mosek requires in seconds
% 
% Author: Christian Kruschel, TU Braunschweig
%         c.kruschel [at] tu-braunschweig.de
%
% !! Program requires Mosek !!


y = [];
norm_y = [];

[m,n] = size(A);
if length(s) ~= n
    error('Size of A and length of s must coincide');
end

I = find(abs(s)==1);
l = length(I);
Ic = setdiff(1:n, I);

sI = s(I);
AI = A(:,I);
AIc = A(:,Ic);

p.echo = 0;
param.MSK_IPAR_PRESOLVE_LINDEP_USE = 'MSK_OFF';
param.MSK_IPAR_INTPNT_BASIS = 'MSK_BI_NEVER';

prob.c = [zeros(m,1); 1];
prob.a = sparse([AIc', ones(n-l,1); 
                 AIc', -ones(n-l,1);
                 AI' , zeros(l, 1)]);
prob.blc = [zeros(n-l,1); -inf*ones(n-l,1); sI];
prob.buc = [inf*ones(n-l,1); zeros(n-l,1);  sI];
prob.blx = [];
prob.bux = [];
    
cmd = 'minimize';
cmd = sprintf('%s echo(%i)',cmd,p.echo);

tic
[~,res] = mosekopt(cmd,prob,param);
time = toc;

status = res.sol.itr.solsta;

if strcmp(res.sol.itr.solsta,'OPTIMAL') || ...
            strcmp(res.sol.itr.solsta,'NEAR_OPTIMAL')
        
        y = res.sol.itr.xx(1:end-1);
        norm_y = res.sol.itr.xx(end);
end
