function [x,norm,status,time] = solve_l1_minimization(A,b)
% Function solve_l1_minimization
% delivers x satisyfing
% min_y norm(y,1) subject to A*y = b,
% using Mosek through linear optimization.
%
% Input:  A	: Matrix of l1 minimization problem
%	  b	: Right-hand side
% Output: x	: Argmin
% 	  norm  : Min
% 	  R	: Structure of output from Mosek
%	  time	: Time Mosek requires in seconds
%
% !! Program requires Mosek !!


A        = sparse(A);
[m,n]    = size(A);
In       = speye(n);
en       = ones(n,1);
subtract = [];
p.echo = 0;


cmd = 'minimize';
cmd = sprintf('%s echo(%i)',cmd,p.echo);

prob.a   = [A,sparse(m,n); In,-In;-In,-In];
prob.buc = [b;zeros(2*n,1)];
prob.blc = [b;-inf*ones(2*n,1)];
prob.bux = [];
prob.blx = [];
prob.c   = [zeros(n,1);en];


Ctol = 1e-8;

param.MSK_DPAR_INTPNT_TOL_PFEAS = Ctol;
param.MSK_DPAR_INTPNT_TOL_DFEAS = Ctol;
param.MSK_DPAR_INTPNT_TOL_REL_GAP = Ctol;
param.MSK_DPAR_INTPNT_TOL_INFEAS = Ctol;
param.MSK_DPAR_INTPNT_TOL_MU_RED = Ctol;
if m >= n
    param.MSK_IPAR_PRESOLVE_LINDEP_USE = 'MSK_OFF';
end

tic
[r,res] = mosekopt(cmd,prob,param);
time = toc;

xx      = res.sol.itr.xx(1:n);
norm      = res.sol.itr.pobjval;

status = res.sol.itr.solsta;
x      = xx;